/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.1 Stauverlaufsanalyse
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.anastau.bestimmungstau.indikator;

import java.util.Collection;

import de.bsvrz.ibv.anastau.bestimmungstau.parameter.ISobParameterObserver;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.Strasse;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.VerkehrModellNetz;

/**
 * Das Interface zum Zugriff auf eine Stra&szlig;endefinition innerhalb der
 * Stauobjektermittlung.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public interface ISobStrasse {

	/**
	 * liefert die Fahrtrichtungen, für die Stauobjekte bestimmt werden sollen.
	 *
	 * @return die Liste der Fahrtrichtungen
	 */
	Collection<FahrtRichtung> getFahrtRichtungen();

	/**
	 * liefert die Verwaltung für die Parameter der Stauobjektbestimmung für die
	 * Stra&szlig;e.
	 *
	 * @return die Parameterverwaltung
	 */
	ISobParameterObserver getParameter();

	/**
	 * liefert das Systemobjekt aus der Datenverteilerkonfiguration, das die
	 * Stra&szlig;e repr&auml;sentiert.
	 *
	 * @return das Objekt
	 */
	Strasse getStrasse();

	/**
	 * liefert das VerkehsmodellNetz, aus dem die Stra&szlig;e ermittelt wurde.
	 *
	 * @return das Netz
	 */
	VerkehrModellNetz getNetz();
}
