/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.1 Stauverlaufsanalyse
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.anastau.migration.modell.verkehr;

import java.util.Arrays;
import java.util.Collection;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ibv.anastau.migration.modell.ModellObjektFactory;
import de.bsvrz.ibv.anastau.migration.modell.SystemObjekt;
import de.bsvrz.ibv.anastau.migration.modell.SystemObjektTyp;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.AeusseresStrassenSegment;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.FahrStreifen;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.InneresStrassenSegment;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.MessQuerschnitt;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.MessQuerschnittVirtuell;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.Netz;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.Stau;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.Strasse;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.StrassenKnoten;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.StrassenSegment;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.StrassenTeilSegment;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.VerkehrModellNetz;

/**
 * Fabrikmethode f&uuml;r gekapselte Systemobjekte aus dem Verkehrsmodell. Jedes
 * gekapselte Objekt wird als Singleton behandelt und zwischengespeichert.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public class VerkehrsobjektFactory implements ModellObjektFactory {

	@Override
	public SystemObjekt getModellobjekt(final SystemObject objekt) {
		if (objekt == null) {
			throw new IllegalArgumentException(
					"Argument darf nicht null sein.");
		}

		SystemObjekt obj = null;
		if (objekt.isOfType(VerkehrsModellTypen.VERKEHRSMODELLNETZ.getPid())) {
			obj = new VerkehrModellNetz(objekt);
		} else if (objekt.isOfType(VerkehrsModellTypen.NETZ.getPid())) {
			obj = new Netz(objekt);
		} else
			if (objekt.isOfType(VerkehrsModellTypen.MESSQUERSCHNITT.getPid())) {
				obj = new MessQuerschnitt(objekt);
			} else if (objekt.isOfType(
					VerkehrsModellTypen.MESSQUERSCHNITTVIRTUELL.getPid())) {
				obj = new MessQuerschnittVirtuell(objekt);
			} else if (objekt.isOfType(
					VerkehrsModellTypen.AUESSERES_STRASSENSEGMENT.getPid())) {
				obj = new AeusseresStrassenSegment(objekt);
			} else if (objekt.isOfType(
					VerkehrsModellTypen.INNERES_STRASSENSEGMENT.getPid())) {
				obj = new InneresStrassenSegment(objekt);
			} else if (objekt
					.isOfType(VerkehrsModellTypen.STRASSENTEILSEGMENT.getPid())) {
				obj = new StrassenTeilSegment(objekt);
			} else
				if (objekt.isOfType(VerkehrsModellTypen.STRASSENSEGMENT.getPid())) {
					obj = new StrassenSegment(objekt);
				} else if (objekt
						.isOfType(VerkehrsModellTypen.STRASSENKNOTEN.getPid())) {
					obj = new StrassenKnoten(objekt);
				} else if (objekt.isOfType(VerkehrsModellTypen.STRASSE.getPid())) {
					obj = new Strasse(objekt);
				} else if (objekt.isOfType(VerkehrsModellTypen.STAU.getPid())) {
					obj = new Stau(objekt);
				} else if (objekt.isOfType(VerkehrsModellTypen.FAHRSTREIFEN.getPid())) {
					obj = new FahrStreifen(objekt);
				}

		return obj;
	}

	@Override
	public Collection<? extends SystemObjektTyp> getTypen() {
		return Arrays.asList(VerkehrsModellTypen.values());
	}
}
