/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.1 Stauverlaufsanalyse
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.anastau.progstau;

import de.bsvrz.ibv.anastau.anastau.StauverlaufsAnalyse;
import de.bsvrz.ibv.anastau.bestimmungstau.stau.IStau;
import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.VerkehrModellNetz;

/**
 * Interface für ein Modul zur Prognose des Verlaufs eines Stauobjekts.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public interface IStauverlaufsPrognose {

	/**
	 * initialisiert das Modul mit der verwendeten Datenverteilerverbindung für
	 * das übergebene VerkehrsmodellNetz.
	 *
	 * @param applikation
	 *            die Applikation zur Stauverlaufsprognose
	 * @param netz
	 *            das verwendete Netz
	 */
	void init(StauverlaufsAnalyse applikation, VerkehrModellNetz netz);

	/**
	 * liefert die Prognosedaten für den übergebenen Stau.
	 *
	 * @param stau
	 *            der Stau
	 * @param zeitpunkt
	 *            der Zeitpunkt für den die Prognose erfolgen soll
	 * @return die Prognosedaten
	 */
	SvpPrognoseDaten prognose(IStau stau, long zeitpunkt);
}
